@@ -16,7 +16,7 @@
 #ifndef LISTEN_H
 #define LISTEN_H
 /**
- * $Id: a82b91d2962eaec079faf7ed7a5d31086e381400 $
+ * $Id: 1c57512caaa42a6c31f2494f0fcb274f86cfd8c9 $
  *
  * @file listen.h
  * @brief The listener API.
@@ -43,13 +43,13 @@ typedef enum RAD_LISTEN_TYPE {
 } RAD_LISTEN_TYPE;
 
 typedef enum RAD_LISTEN_STATUS {
-	RAD_LISTEN_STATUS_INIT = 0,
-	RAD_LISTEN_STATUS_KNOWN,
-	RAD_LISTEN_STATUS_PAUSE,
-	RAD_LISTEN_STATUS_RESUME,
-	RAD_LISTEN_STATUS_FROZEN,
-	RAD_LISTEN_STATUS_EOL,
-	RAD_LISTEN_STATUS_REMOVE_NOW
+	RAD_LISTEN_STATUS_INIT = 0,		//!< starting up
+	RAD_LISTEN_STATUS_KNOWN,		//!< alive and operating normally
+	RAD_LISTEN_STATUS_PAUSE,		//!< TLS connection checking: don't read normal packets
+	RAD_LISTEN_STATUS_RESUME,		//!< TLS connection checking: resume reading normal packets
+	RAD_LISTEN_STATUS_FROZEN,		//!< alive, but we're not sending any more packets to it
+	RAD_LISTEN_STATUS_EOL,			//!< we're trying to delete it.
+	RAD_LISTEN_STATUS_REMOVE_NOW		//!< no request is using it, delete the listener.
 } RAD_LISTEN_STATUS;
 
 typedef struct rad_listen rad_listen_t;
@@ -187,6 +187,8 @@ typedef struct listen_socket_t {
 	pthread_mutex_t mutex;
 	uint8_t		*data;
 	size_t		partial;
+
+	fr_event_fd_handler_t write_handler;
 	enum {
 		LISTEN_TLS_INIT = 0,
 		LISTEN_TLS_CHECKING,
@@ -194,6 +196,8 @@ typedef struct listen_socket_t {
 		LISTEN_TLS_RUNNING,
 	} state;
 
+	bool		client_closed;
+
 #ifdef WITH_RADIUSV11
 	bool		alpn_checked;
 	bool		radiusv11;		//!< defaults to "no"!
