@@ -11,7 +11,7 @@
  *  rather than being able to move 1/2 of the entries in the chain with
  *  one update.
  *
- * Version:	$Id: d509388841e9194c5228d4bcb0a420d6d009efa9 $
+ * Version:	$Id: 9efff6e825b7bf4eb9ba2e0a7825c5729d957bd7 $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Lesser General Public
@@ -30,7 +30,7 @@
  *  Copyright 2005,2006  The FreeRADIUS server project
  */
 
-RCSID("$Id: d509388841e9194c5228d4bcb0a420d6d009efa9 $")
+RCSID("$Id: 9efff6e825b7bf4eb9ba2e0a7825c5729d957bd7 $")
 
 #include <freeradius-devel/libradius.h>
 
@@ -40,12 +40,12 @@ RCSID("$Id: d509388841e9194c5228d4bcb0a420d6d009efa9 $")
  */
 #define FR_HASH_NUM_BUCKETS (64)
 
-typedef struct fr_hash_entry_t {
-	struct fr_hash_entry_t *next;
+struct fr_hash_entry_s {
+	fr_hash_entry_t *next;
 	uint32_t	reversed;
 	uint32_t	key;
 	void const 	*data;
-} fr_hash_entry_t;
+};
 
 
 struct fr_hash_table_t {
@@ -150,10 +150,17 @@ static uint8_t parent_byte[256] = {
  */
 static uint32_t reverse(uint32_t key)
 {
-	return ((reversed_byte[key & 0xff] << 24) |
-		(reversed_byte[(key >> 8) & 0xff] << 16) |
-		(reversed_byte[(key >> 16) & 0xff] << 8) |
-		(reversed_byte[(key >> 24) & 0xff]));
+	/*
+	 *	Cast to uint32_t is required because the
+	 *	default type of of the expression is an
+	 *	int and ubsan correctly complains that
+	 *	the result of 0xff << 24 won't fit in an
+	 *	unsigned 32bit integer.
+	 */
+	return (((uint32_t)reversed_byte[key & 0xff] << 24) |
+		((uint32_t)reversed_byte[(key >> 8) & 0xff] << 16) |
+		((uint32_t)reversed_byte[(key >> 16) & 0xff] << 8) |
+		((uint32_t)reversed_byte[(key >> 24) & 0xff]));
 }
 
 /*
@@ -620,6 +627,81 @@ int fr_hash_table_walk(fr_hash_table_t *ht,
 	return 0;
 }
 
+/** Iterate over entries in a hash table
+ *
+ * @note If the hash table is modified the iterator should be considered invalidated.
+ *
+ * @param[in] ht	to iterate over.
+ * @param[in] iter	Pointer to an iterator struct, used to maintain
+ *			state between calls.
+ * @return
+ *	- User data.
+ *	- NULL if at the end of the list.
+ */
+void *fr_hash_table_iter_next(fr_hash_table_t *ht, fr_hash_iter_t *iter)
+{
+	fr_hash_entry_t *node;
+	uint32_t	i;
+	void		*out;
+
+	/*
+	 *	Return the next element in the bucket
+	 */
+	if (iter->node != &ht->null) {
+		node = iter->node;
+		iter->node = node->next;
+
+		memcpy(&out, &node->data, sizeof(out)); /* const issues */
+		return out;
+	}
+
+	if (iter->bucket == 0) return NULL;
+
+	/*
+	 *	We might have to go through multiple empty
+	 *	buckets to find one that contains something
+	 *	we should return
+	 */
+	i = iter->bucket - 1;
+	for (;;) {
+		if (!ht->buckets[i]) fr_hash_table_fixup(ht, i);
+
+		node = ht->buckets[i];
+		if (node == &ht->null) {
+			if (i == 0) break;
+			i--;
+			continue;	/* This bucket was empty too... */
+		}
+
+		iter->node = node->next;		/* Store the next one to examine */
+		iter->bucket = i;
+
+		memcpy(&out, &node->data, sizeof(out)); /* const issues */
+		return out;
+	}
+	iter->bucket = i;
+
+	return NULL;
+}
+
+/** Initialise an iterator
+ *
+ * @note If the hash table is modified the iterator should be considered invalidated.
+ *
+ * @param[in] ht	to iterate over.
+ * @param[out] iter	to initialise.
+ * @return
+ *	- The first entry in the hash table.
+ *	- NULL if the hash table is empty.
+ */
+void *fr_hash_table_iter_init(fr_hash_table_t *ht, fr_hash_iter_t *iter)
+{
+	iter->bucket = ht->num_buckets;
+	iter->node = &ht->null;
+
+	return fr_hash_table_iter_next(ht, iter);
+}
+
 
 #ifdef TESTING
 /*
