@@ -49,7 +49,7 @@
 
 */
 
-RCSID("$Id: 4f79a32d64a0ac1388012f7f060347b6d79acddf $")
+RCSID("$Id: 9002464f9e1693ee99707551c4f25603cb1dc3c8 $")
 
 #include "snprintf.h"
 
@@ -313,7 +313,7 @@ double d;
 		PUT_CHAR('0', p); PUT_CHAR(*p->pf, p);
 	}
 	while (*tmp) { /* hexa */
-		PUT_CHAR((*p->pf == 'X' ? toupper(*tmp) : *tmp), p);
+		PUT_CHAR((*p->pf == 'X' ? toupper((uint8_t) *tmp) : *tmp), p);
 		tmp++;
 	}
 	PAD_LEFT(p);
@@ -482,7 +482,7 @@ struct DATA * p;
 			case '1': case '2': case '3':
 			case '4': case '5': case '6':
 			case '7': case '8': case '9':		 /* gob all the digits */
-	for (i = 0; isdigit(*s); i++, s++)
+	for (i = 0; isdigit((uint8_t) *s); i++, s++)
 		if (i < MAX_FIELD/2 - 1)
 			number[i] = *s;
 	number[i] = '\0';
