@@ -1,7 +1,7 @@
 /*
  * value.c	Functions to handle value_data_t
  *
- * Version:	$Id: 5658767fc67d76eb56609f91771013f8b3e8d904 $
+ * Version:	$Id: dddfbeae03a2ba05ae67f55edb9408520dea9553 $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Lesser General Public
@@ -20,7 +20,7 @@
  * Copyright 2014 The FreeRADIUS server project
  */
 
-RCSID("$Id: 5658767fc67d76eb56609f91771013f8b3e8d904 $")
+RCSID("$Id: dddfbeae03a2ba05ae67f55edb9408520dea9553 $")
 
 #include <freeradius-devel/libradius.h>
 #include <ctype.h>
@@ -586,9 +586,7 @@ ssize_t value_data_from_str(TALLOC_CTX *ctx, value_data_t *dst,
 		return -1;
 
 	/* raw octets: 0x01020304... */
-#ifndef WITH_ASCEND_BINARY
 	do_octets:
-#endif
 	case PW_TYPE_OCTETS:
 	{
 		uint8_t	*p;
@@ -628,25 +626,15 @@ ssize_t value_data_from_str(TALLOC_CTX *ctx, value_data_t *dst,
 	case PW_TYPE_ABINARY:
 #ifdef WITH_ASCEND_BINARY
 		if ((len > 1) && (strncasecmp(src, "0x", 2) == 0)) {
-			ssize_t bin;
+			goto do_octets;
 
-			if (len > ((sizeof(dst->filter) + 1) * 2)) {
-				fr_strerror_printf("Hex data is too large for ascend filter");
-				return -1;
-			}
-
-			bin = fr_hex2bin((uint8_t *) &dst->filter, ret, src + 2, len - 2);
-			if (bin < ret) {
-				memset(((uint8_t *) &dst->filter) + bin, 0, ret - bin);
-			}
 		} else {
-			if (ascend_parse_filter(dst, src, len) < 0 ) {
+			ret = ascend_parse_filter(ctx, dst, src, len);
+			if (ret < 0) {
 				/* Allow ascend_parse_filter's strerror to bubble up */
 				return -1;
 			}
 		}
-
-		ret = sizeof(dst->filter);
 		goto finish;
 #else
 		/*
@@ -863,8 +851,23 @@ ssize_t value_data_from_str(TALLOC_CTX *ctx, value_data_t *dst,
 		 *	intermediary variable to handle the conversions.
 		 */
 		time_t date;
+		struct tm tm = { 0 };
+		char *end;
 
-		if (fr_get_time(src, &date) < 0) {
+		/*
+		 *	Try to parse dates via locale-specific names,
+		 *	using the same format string as strftime(),
+		 *	below.
+		 *
+		 *	If that fails (e.g. unix dates as integer),
+		 *	then we fall back to our parsing routine,
+		 *	which is much more forgiving.
+		 */
+		end = strptime(src, "%b %e %Y %H:%M:%S %Z", &tm);
+		if (end && (*end == '\0')) {
+			date = mktime(&tm);
+
+		} else if (fr_get_time(src, &date) < 0) {
 			fr_strerror_printf("failed to parse time string \"%s\"", src);
 			return -1;
 		}
@@ -903,11 +906,11 @@ ssize_t value_data_from_str(TALLOC_CTX *ctx, value_data_t *dst,
 		while (*cp) {
 			if (cp[1] == ':') {
 				c1 = hextab;
-				c2 = memchr(hextab, tolower((int) cp[0]), 16);
+				c2 = memchr(hextab, tolower((uint8_t) cp[0]), 16);
 				cp += 2;
 			} else if ((cp[1] != '\0') && ((cp[2] == ':') || (cp[2] == '\0'))) {
-				c1 = memchr(hextab, tolower((int) cp[0]), 16);
-				c2 = memchr(hextab, tolower((int) cp[1]), 16);
+				c1 = memchr(hextab, tolower((uint8_t) cp[0]), 16);
+				c2 = memchr(hextab, tolower((uint8_t) cp[1]), 16);
 				cp += 2;
 				if (*cp == ':') cp++;
 			} else {
@@ -982,7 +985,11 @@ static ssize_t value_data_hton(value_data_t *dst, PW_TYPE dst_type, void const *
 	case PW_TYPE_INTEGER64:
 		dst_len = sizeof(dst->integer64);
 
-		if (src_len < dst_len) return -1;
+		if (src_len < dst_len) {
+		too_small:
+			fr_strerror_printf("Source is too small to cast to destination type");
+			return -1;
+		}
 
 		dst->integer64 = htonll(*(uint64_t const *)src);
 		break;
@@ -993,7 +1000,7 @@ static ssize_t value_data_hton(value_data_t *dst, PW_TYPE dst_type, void const *
 	case PW_TYPE_SIGNED:
 		dst_len = sizeof(dst->integer);
 
-		if (src_len < dst_len) return -1;
+		if (src_len < dst_len) goto too_small;
 
 		dst->integer = htonl(*(uint32_t const *)src);
 		break;
@@ -1002,7 +1009,7 @@ static ssize_t value_data_hton(value_data_t *dst, PW_TYPE dst_type, void const *
 	case PW_TYPE_SHORT:
 		dst_len = sizeof(dst->ushort);
 
-		if (src_len < dst_len) return -1;
+		if (src_len < dst_len) goto too_small;
 
 		dst->ushort = htons(*(uint16_t const *)src);
 		break;
@@ -1011,7 +1018,7 @@ static ssize_t value_data_hton(value_data_t *dst, PW_TYPE dst_type, void const *
 	case PW_TYPE_BYTE:
 		dst_len = sizeof(dst->byte);
 
-		if (src_len < dst_len) return -1;
+		if (src_len < dst_len) goto too_small;
 
 		dst->byte = *(uint8_t const *)src;
 		break;
@@ -1024,7 +1031,7 @@ static ssize_t value_data_hton(value_data_t *dst, PW_TYPE dst_type, void const *
 		/*
 		 *	Not enough information, die.
 		 */
-		if (src_len < dst_len) return -1;
+		if (src_len < dst_len) goto too_small;
 
 		/*
 		 *	Copy only as much as we need from the source.
@@ -1032,20 +1039,6 @@ static ssize_t value_data_hton(value_data_t *dst, PW_TYPE dst_type, void const *
 		memcpy(dst_ptr, src, dst_len);
 		break;
 
-	case PW_TYPE_ABINARY:
-		dst_len = sizeof(dst->filter);
-		dst_ptr = (uint8_t *) dst->filter;
-
-		/*
-		 *	Too little data is OK here.
-		 */
-		if (src_len < dst_len) {
-			memcpy(dst_ptr, src, src_len);
-			memset(dst_ptr + src_len, 0, dst_len - src_len);			
-			break;
-		}
-		goto copy;
-
 	case PW_TYPE_IFID:
 		dst_len = sizeof(dst->ifid);
 		dst_ptr = (uint8_t *) dst->ifid;
@@ -1060,7 +1053,7 @@ static ssize_t value_data_hton(value_data_t *dst, PW_TYPE dst_type, void const *
 		dst_len = sizeof(dst->ipv4prefix);
 		dst_ptr = (uint8_t *) dst->ipv4prefix;
 
-		if (src_len < dst_len) return -1;
+		if (src_len < dst_len) goto too_small;
 		if ((((uint8_t const *)src)[1] & 0x3f) > 32) return -1;
 		goto copy;
 
@@ -1072,7 +1065,7 @@ static ssize_t value_data_hton(value_data_t *dst, PW_TYPE dst_type, void const *
 		 *	Smaller IPv6 prefixes are OK, too, so long as
 		 *	they're not too short.
 		 */
-		if (src_len < 2) return -1;
+		if (src_len < 2) goto too_small;
 
 		/*
 		 *	Prefix is too long.
@@ -1093,6 +1086,8 @@ static ssize_t value_data_hton(value_data_t *dst, PW_TYPE dst_type, void const *
 		goto copy;
 
 	default:
+		fr_strerror_printf("Invalid cast to %s",
+				   fr_int2str(dict_attr_types, dst_type, "<INVALID>"));
 		return -1;	/* can't do it */
 	}
 
@@ -1120,7 +1115,10 @@ ssize_t value_data_cast(TALLOC_CTX *ctx, value_data_t *dst,
 {
 	ssize_t dst_len;
 
-	if (!fr_assert(dst_type != src_type)) return -1;
+	if (!fr_assert(dst_type != src_type)) {
+		fr_strerror_printf("Types do not match");
+		return -1;
+	}
 
 	/*
 	 *	Deserialise a value_data_t
@@ -1129,14 +1127,26 @@ ssize_t value_data_cast(TALLOC_CTX *ctx, value_data_t *dst,
 		return value_data_from_str(ctx, dst, &dst_type, dst_enumv, src->strvalue, src_len, '\0');
 	}
 
+	if (dst_type == PW_TYPE_ABINARY) {
+		if (src_type != PW_TYPE_OCTETS) goto invalid_cast;
+
+		dst->filter = talloc_memdup(ctx, src->octets, src_len);
+		return src_len;
+	}
+
 	/*
 	 *	Converts the src data to octets with no processing.
 	 */
 	if (dst_type == PW_TYPE_OCTETS) {
-		dst_len = value_data_hton(dst, src_type, src, src_len);
-		if (dst_len < 0) return -1;
+		if (src_type != PW_TYPE_ABINARY) {
+			dst_len = value_data_hton(dst, src_type, src, src_len);
+			if (dst_len < 0) return -1;
 
-		dst->octets = talloc_memdup(ctx, dst, dst_len);
+			dst->octets = talloc_memdup(ctx, dst, dst_len);
+		} else {
+			dst->octets = talloc_memdup(ctx, src->filter, src_len);
+			dst_len = src_len;
+		}
 		talloc_set_type(dst->octets, uint8_t);
 		return dst_len;
 	}
@@ -1623,9 +1633,9 @@ char *value_data_aprints(TALLOC_CTX *ctx,
 
 	case PW_TYPE_ABINARY:
 #ifdef WITH_ASCEND_BINARY
-		p = talloc_array(ctx, char, 128);
+		p = talloc_array(ctx, char, 256);
 		if (!p) return NULL;
-		print_abinary(p, 128, (uint8_t const *) &data->filter, inlen, 0);
+		print_abinary(p, 256, data->filter, inlen, 0);
 		break;
 #else
 		  /* FALL THROUGH */
@@ -1836,7 +1846,7 @@ print_int:
 
 	case PW_TYPE_ABINARY:
 #ifdef WITH_ASCEND_BINARY
-		print_abinary(buf, sizeof(buf), (uint8_t const *) data->filter, inlen, quote);
+		print_abinary(buf, sizeof(buf), data->filter, inlen, quote);
 		a = buf;
 		len = strlen(buf);
 		break;
