@@ -1,7 +1,7 @@
 /*
  * command.c	Command socket processing.
  *
- * Version:	$Id: 988f43bb02451d9ad744d23bc60fac69f2a92e13 $
+ * Version:	$Id: 1e5d43212283d80ede908200fae86f49b54d9afb $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -2701,7 +2701,7 @@ static int command_stats_socket(rad_listen_t *listener, int argc, char *argv[])
 	return command_print_stats(listener, &sock->stats, auth, 0);
 }
 
-static int command_stats_pool(rad_listen_t *listener, UNUSED int argc, UNUSED char *argv[])
+static int command_stats_pool(rad_listen_t *listener, int argc, char *argv[])
 {
 	CONF_SECTION *cs;
 	module_instance_t *mi;
@@ -2805,7 +2805,7 @@ static int command_del_home_server(rad_listen_t *listener, int argc, char *argv[
 		return 0;
 	}
 
-	if (home_server_delete(argv[0], argv[1]) < 0) {
+	if (home_server_delete_byname(argv[0], argv[1]) < 0) {
 		cprintf_error(listener, "Failed deleted home_server %s - %s\n", argv[1], fr_strerror());
 		return 0;
 	}
