@@ -5,7 +5,7 @@
  *		write a decent parser. I know how to do that, really :)
  *		miquels@cistron.nl
  *
- * Version:	$Id: 75bb5d616e1ea7b053e5951e1d55c23920c20400 $
+ * Version:	$Id: bc8969f37aa8d384ef858d0d67e86caf04c37e5d $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
 
-RCSID("$Id: 75bb5d616e1ea7b053e5951e1d55c23920c20400 $")
+RCSID("$Id: bc8969f37aa8d384ef858d0d67e86caf04c37e5d $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/parser.h>
@@ -2632,6 +2632,19 @@ static int cf_section_read(char const *filename, int *lineno, FILE *fp,
 					}
 					if (*p != '\0') continue;
 
+					/*
+					 *	Ignore config files generated by deb / rpm packaging updates.
+					 */
+					len = strlen(dp->d_name);
+					if ((len > 10) && (strncmp(&dp->d_name[len - 10], ".dpkg-dist", 10) == 0)) {
+					pkg_file:
+						WARN("Ignoring packaging system produced file %s%s", value, dp->d_name);
+					 	continue;
+					}
+					if ((len > 9) && (strncmp(&dp->d_name[len - 9], ".dpkg-old", 9) == 0)) goto pkg_file;
+					if ((len > 7) && (strncmp(&dp->d_name[len - 7], ".rpmnew", 9) == 0)) goto pkg_file;
+					if ((len > 8) && (strncmp(&dp->d_name[len - 8], ".rpmsave", 10) == 0)) goto pkg_file;
+
 					slen = snprintf(buf2, sizeof(buf2), "%s%s",
 							value, dp->d_name);
 					if (slen >= (int) sizeof(buf2) || slen < 0) {
