@@ -15,7 +15,7 @@
  */
 
 /*
- * $Id: 34683a2e1295282d2d6ae05e360412df6d83c667 $
+ * $Id: e8da0be78c408b580a83aa6f367cbfce27fe2d44 $
  *
  * @brief map / template functions
  * @file main/map.c
@@ -26,7 +26,7 @@
  * @copyright 2013  Alan DeKok <aland@freeradius.org>
  */
 
-RCSID("$Id: 34683a2e1295282d2d6ae05e360412df6d83c667 $")
+RCSID("$Id: e8da0be78c408b580a83aa6f367cbfce27fe2d44 $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/rad_assert.h>
@@ -271,6 +271,11 @@ int map_afrom_cp(TALLOC_CTX *ctx, vp_map_t **out, CONF_PAIR *cp,
 			goto error;
 		}
 
+		if ((map->lhs->tmpl_tag == TAG_VALUE) &&
+		    ((map->op == T_OP_CMP_FALSE) || (map->op == T_OP_CMP_TRUE) || (map->op == T_OP_SUB))) {
+			cf_log_err_cp(cp, "Cannot use ':V' for this operator");
+			goto error;
+		}
 		break;
 	}
 
@@ -297,6 +302,24 @@ int map_afrom_cp(TALLOC_CTX *ctx, vp_map_t **out, CONF_PAIR *cp,
 		goto error;
 	}
 
+	/*
+	 *	For :V, enforce / verify limits on the RHS.
+	 */
+	if ((map->lhs->type == TMPL_TYPE_ATTR) && (map->lhs->tmpl_tag == TAG_VALUE)) {
+		fr_assert(map->lhs->tmpl_da->flags.has_tag);
+
+		switch (map->rhs->type) {
+		case TMPL_TYPE_XLAT_STRUCT:
+		case TMPL_TYPE_XLAT:
+		case TMPL_TYPE_LITERAL:
+			break;
+
+		default:
+			cf_log_err_cp(cp, "Cannot use ':V' for tags here.");
+			goto error;
+		}
+	}
+
 	if (map->rhs->type == TMPL_TYPE_ATTR) {
 		/*
 		 *	We cannot assign a count to an attribute.  That must
@@ -725,8 +748,10 @@ static int map_exec_to_vp(TALLOC_CTX *ctx, VALUE_PAIR **out, REQUEST *request, v
 
 		vp = fr_pair_afrom_da(ctx, map->lhs->tmpl_da);
 		if (!vp) return -1;
+
 		vp->op = map->op;
 		vp->tag = map->lhs->tmpl_tag;
+
 		if (fr_pair_value_from_str(vp, answer, -1) < 0) {
 			fr_pair_list_free(&vp);
 			return -2;
@@ -835,14 +860,15 @@ int map_to_vp(TALLOC_CTX *ctx, VALUE_PAIR **out, REQUEST *request, vp_map_t cons
 		RDEBUG2("EXPAND %s", map->rhs->name);
 		RDEBUG2("   --> %s", str);
 
+		new->op = map->op;
+		new->tag = map->lhs->tmpl_tag;
+
 		rcode = fr_pair_value_from_str(new, str, -1);
 		talloc_free(str);
 		if (rcode < 0) {
 			fr_pair_list_free(&new);
 			goto error;
 		}
-		new->op = map->op;
-		new->tag = map->lhs->tmpl_tag;
 		*out = new;
 		break;
 
@@ -860,14 +886,15 @@ int map_to_vp(TALLOC_CTX *ctx, VALUE_PAIR **out, REQUEST *request, vp_map_t cons
 			goto error;
 		}
 
+		new->op = map->op;
+		new->tag = map->lhs->tmpl_tag;
+
 		rcode = fr_pair_value_from_str(new, str, -1);
 		talloc_free(str);
 		if (rcode < 0) {
 			fr_pair_list_free(&new);
 			goto error;
 		}
-		new->op = map->op;
-		new->tag = map->lhs->tmpl_tag;
 		*out = new;
 		break;
 
@@ -878,12 +905,13 @@ int map_to_vp(TALLOC_CTX *ctx, VALUE_PAIR **out, REQUEST *request, vp_map_t cons
 		new = fr_pair_afrom_da(ctx, map->lhs->tmpl_da);
 		if (!new) return -1;
 
+		new->op = map->op;
+		new->tag = map->lhs->tmpl_tag;
+
 		if (fr_pair_value_from_str(new, map->rhs->name, -1) < 0) {
 			rcode = 0;
 			goto error;
 		}
-		new->op = map->op;
-		new->tag = map->lhs->tmpl_tag;
 		*out = new;
 		break;
 
@@ -914,6 +942,10 @@ int map_to_vp(TALLOC_CTX *ctx, VALUE_PAIR **out, REQUEST *request, vp_map_t cons
 				new = fr_pair_afrom_da(ctx, map->lhs->tmpl_da);
 				if (!new) return -1;
 
+				new->op = map->op;
+				new->tag = map->lhs->tmpl_tag;
+				fr_assert(new->tag != TAG_VALUE);
+
 				len = value_data_cast(new, &new->data, new->da->type, new->da,
 						      vp->da->type, vp->da, &vp->data, vp->vp_length);
 				if (len < 0) {
@@ -931,8 +963,6 @@ int map_to_vp(TALLOC_CTX *ctx, VALUE_PAIR **out, REQUEST *request, vp_map_t cons
 					rad_assert(new->vp_strvalue != NULL);
 				}
 
-				new->op = map->op;
-				new->tag = map->lhs->tmpl_tag;
 				fr_cursor_insert(&to, new);
 			}
 			return 0;
@@ -942,6 +972,7 @@ int map_to_vp(TALLOC_CTX *ctx, VALUE_PAIR **out, REQUEST *request, vp_map_t cons
 		 *   Otherwise we just need to fixup the attribute types
 		 *   and operators
 		 */
+		fr_assert(map->lhs->tmpl_tag != TAG_VALUE);
 		for (; vp; vp = fr_cursor_next(&from)) {
 			vp->da = map->lhs->tmpl_da;
 			vp->op = map->op;
@@ -959,15 +990,16 @@ int map_to_vp(TALLOC_CTX *ctx, VALUE_PAIR **out, REQUEST *request, vp_map_t cons
 		new = fr_pair_afrom_da(ctx, map->lhs->tmpl_da);
 		if (!new) return -1;
 
+		new->op = map->op;
+		new->tag = map->lhs->tmpl_tag;
+		fr_assert(new->tag != TAG_VALUE);
+
 		len = value_data_copy(new, &new->data, new->da->type, &map->rhs->tmpl_data_value,
 				      map->rhs->tmpl_data_length);
 		if (len < 0) goto error;
 
 		new->vp_length = len;
-		new->op = map->op;
-		new->tag = map->lhs->tmpl_tag;
 		*out = new;
-
 		VERIFY_MAP(map);
 		break;
 
@@ -1331,7 +1363,7 @@ int map_to_request(REQUEST *request, vp_map_t const *map, radius_map_getvalue_t
 	/*
 	 *	Another fixup pass to set tags on attributes were about to insert
 	 */
-	if (map->lhs->tmpl_tag != TAG_ANY) {
+	if (TAG_VALID(map->lhs->tmpl_tag)) {
 		for (vp = fr_cursor_init(&src_list, &head);
 		     vp;
 		     vp = fr_cursor_next(&src_list)) {
