@@ -1,7 +1,7 @@
 /*
  * eap.c    rfc2284 & rfc2869 implementation
  *
- * Version:     $Id: 1ece323072c7b2bdb615885fa5a5c4179cee25bb $
+ * Version:     $Id: 08b7f539f7364e74ab7cee9fc8fe20b6027f066a $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -56,7 +56,7 @@
 
 #include <freeradius-devel/modpriv.h>
 
-RCSID("$Id: 1ece323072c7b2bdb615885fa5a5c4179cee25bb $")
+RCSID("$Id: 08b7f539f7364e74ab7cee9fc8fe20b6027f066a $")
 
 #include "rlm_eap.h"
 #include <ctype.h>
@@ -382,18 +382,24 @@ eap_rcode_t eap_method_select(rlm_eap_t *inst, eap_handler_t *handler)
 		 */
 		vp = fr_pair_find_by_num(handler->request->config, PW_EAP_TYPE, 0,
 			      TAG_ANY);
-		if (vp) next = vp->vp_integer;
+		if (vp) {
+			next = vp->vp_integer;
 
-		/*
-		 *	Ensure it's valid.
-		 */
-		if ((next < PW_EAP_MD5) ||
-		    (next >= PW_EAP_MAX_TYPES) ||
-		    (!inst->methods[next])) {
-			REDEBUG2("Tried to start unsupported EAP type %s (%d)",
-				 eap_type2name(next), next);
+			/*
+			 *	Ensure it's valid.
+			 */
+			if ((next < PW_EAP_MD5) ||
+			    (next >= PW_EAP_MAX_TYPES) ||
+			    (!inst->methods[next])) {
+				REDEBUG2("Tried to start unsupported EAP type %s (%d)",
+					 eap_type2name(next), next);
 
-			return EAP_INVALID;
+				return EAP_INVALID;
+			}
+
+			RDEBUG("Found &control:EAP-Type = %s", eap_type2name(next));
+		} else {
+			RDEBUG("Using default_eap_type = %s", eap_type2name(next));
 		}
 
 	do_initiate:
@@ -442,6 +448,7 @@ eap_rcode_t eap_method_select(rlm_eap_t *inst, eap_handler_t *handler)
 			return EAP_INVALID;
 		}
 
+		RDEBUG("Found compatible type in NAK - EAP-Type = %s", eap_type2name(next));
 		goto do_initiate;
 
 		/*
@@ -803,7 +810,7 @@ int eap_start(rlm_eap_t *inst, REQUEST *request)
 	    ((eap_msg->vp_octets[4] == 0) ||
 	     (eap_msg->vp_octets[4] >= PW_EAP_MAX_TYPES) ||
 	     (!inst->methods[eap_msg->vp_octets[4]]))) {
-		RDEBUG2("Ignoring Unknown EAP type");
+		RDEBUG2("Ignoring Unknown EAP type %02x", eap_msg->vp_octets[4]);
 		return EAP_NOOP;
 	}
 
@@ -833,6 +840,8 @@ int eap_start(rlm_eap_t *inst, REQUEST *request)
 	}
 
 	if ((eap_msg->vp_octets[4] == PW_EAP_TTLS) ||
+	    (eap_msg->vp_octets[4] == PW_EAP_FAST) ||
+	    (eap_msg->vp_octets[4] == PW_EAP_TEAP) ||
 	    (eap_msg->vp_octets[4] == PW_EAP_PEAP)) {
 		RDEBUG2("Continuing tunnel setup");
 		return EAP_OK;
@@ -1214,7 +1223,7 @@ eap_handler_t *eap_handler(rlm_eap_t *inst, eap_packet_raw_t **eap_packet_p,
 		       }
 	       }
 	} else {		/* packet was EAP identity */
-		handler = eap_handler_alloc(inst);
+		handler = eap_handler_alloc(inst, request);
 		if (!handler) {
 			goto error;
 		}
@@ -1224,10 +1233,14 @@ eap_handler_t *eap_handler(rlm_eap_t *inst, eap_packet_raw_t **eap_packet_p,
 		 */
 		handler->identity = eap_identity(request, handler, eap_packet);
 		if (!handler->identity) {
-			RDEBUG("Identity Unknown, authentication failed");
-		error2:
-			talloc_free(handler);
-			goto error;
+			if (!inst->allow_empty_identities) {
+				RDEBUG("Identity Unknown, authentication failed");
+			error2:
+				talloc_free(handler);
+				goto error;
+			}
+
+			handler->identity = "";
 		}
 
 	       vp = fr_pair_find_by_num(request->packet->vps, PW_USER_NAME, 0, TAG_ANY);
